@@ -109,16 +109,18 @@ where
                 write!(w, ")").unwrap();
                 Ok(())
             }
-            In(col, vals) => write!(
-                w,
-                "{} IN ({})",
-                col,
-                vals.iter()
-                    .map(|v| v.to_string())
-                    .collect::<Vec<String>>()
-                    .as_slice()
-                    .join(", ")
-            ),
+            In(col, vals) => {
+                write!(w, "{} IN (", col).unwrap();
+                let mut remaining = vals.len();
+                for val in vals {
+                    f(Expr::Val(val), values, pls, w);
+                    if remaining > 1 {
+                        write!(w, ", ").unwrap();
+                        remaining -= 1;
+                    }
+                }
+                write!(w, ")")
+            }
         },
     }
     .unwrap()
@@ -136,14 +138,19 @@ pub fn sql_insert_with_placeholders(
     pls: &mut impl PlaceholderSource,
     w: &mut impl Write,
 ) {
-    write!(w, "INSERT INTO {} (", table).unwrap();
-    list_columns(columns, w);
-    write!(w, ") VALUES (").unwrap();
-    columns.iter().fold("", |sep, _| {
-        write!(w, "{}{}", sep, pls.next_placeholder()).unwrap();
-        ", "
-    });
-    write!(w, ")").unwrap();
+    write!(w, "INSERT INTO {} ", table).unwrap();
+    if !columns.is_empty() {
+        write!(w, "(").unwrap();
+        list_columns(columns, w);
+        write!(w, ") VALUES (").unwrap();
+        columns.iter().fold("", |sep, _| {
+            write!(w, "{}{}", sep, pls.next_placeholder()).unwrap();
+            ", "
+        });
+        write!(w, ")").unwrap();
+    } else {
+        write!(w, "DEFAULT VALUES ").unwrap();
+    }
 }
 
 pub fn sql_update_with_placeholders(
@@ -165,6 +172,10 @@ pub fn sql_limit(limit: i32, w: &mut impl Write) {
     write!(w, " LIMIT {}", limit).unwrap();
 }
 
+pub fn sql_offset(offset: i32, w: &mut impl Write) {
+    write!(w, " OFFSET {}", offset).unwrap();
+}
+
 pub fn sql_order(order: &[Order], w: &mut impl Write) {
     write!(w, " ORDER BY ").unwrap();
     order.iter().fold("", |sep, o| {
