@@ -11,10 +11,11 @@ macro_rules! connection_method_wrapper {
                 columns: &'b [Column],
                 expr: Option<BoolExpr>,
                 limit: Option<i32>,
+                offset: Option<i32>,
                 sort: Option<&[crate::query::Order]>,
             ) -> Result<RawQueryResult<'a>> {
                 self.wrapped_connection_methods()?
-                    .query(table, columns, expr, limit, sort)
+                    .query(table, columns, expr, limit, offset, sort)
             }
             fn insert_returning_pk(
                 &self,
