@@ -39,7 +39,7 @@ impl Backend for PgBackend {
         let mut current: ADB = (*current).clone();
         Ok(ops
             .iter()
-            .map(|o| sql_for_op(&mut current, &o))
+            .map(|o| sql_for_op(&mut current, o))
             .collect::<Result<Vec<String>>>()?
             .join("\n"))
     }
@@ -131,6 +131,7 @@ where
         columns: &'b [Column],
         expr: Option<BoolExpr>,
         limit: Option<i32>,
+        offset: Option<i32>,
         order: Option<&[query::Order]>,
     ) -> Result<RawQueryResult<'a>> {
         let mut sqlquery = String::new();
@@ -154,6 +155,10 @@ where
             helper::sql_limit(limit, &mut sqlquery)
         }
 
+        if let Some(offset) = offset {
+            helper::sql_offset(offset, &mut sqlquery)
+        }
+
         if cfg!(feature = "log") {
             debug!("query sql {}", sqlquery);
         }
@@ -378,11 +383,11 @@ impl<'a> postgres::types::ToSql for SqlValRef<'a> {
             Timestamp(dt) => dt.to_sql_checked(requested_ty, out),
             Null => Ok(postgres::types::IsNull::Yes),
             Custom(SqlValRefCustom::PgToSql { ty, tosql }) => {
-                check_type_match(&ty, &requested_ty)?;
+                check_type_match(ty, requested_ty)?;
                 tosql.to_sql_checked(requested_ty, out)
             }
             Custom(SqlValRefCustom::PgBytes { ty, data }) => {
-                check_type_match(&ty, &requested_ty)?;
+                check_type_match(ty, requested_ty)?;
                 out.put(*data);
                 Ok(postgres::types::IsNull::No)
             }
@@ -507,22 +512,27 @@ where
 
 fn sql_for_op(current: &mut ADB, op: &Operation) -> Result<String> {
     match op {
-        Operation::AddTable(table) => Ok(create_table(&table)?),
-        Operation::RemoveTable(name) => Ok(drop_table(&name)),
-        Operation::AddColumn(tbl, col) => add_column(&tbl, &col),
-        Operation::RemoveColumn(tbl, name) => Ok(remove_column(&tbl, &name)),
-        Operation::ChangeColumn(tbl, old, new) => change_column(current, &tbl, &old, Some(new)),
+        Operation::AddTable(table) => Ok(create_table(table, false)?),
+        Operation::AddTableIfNotExists(table) => Ok(create_table(table, true)?),
+        Operation::RemoveTable(name) => Ok(drop_table(name)),
+        Operation::AddColumn(tbl, col) => add_column(tbl, col),
+        Operation::RemoveColumn(tbl, name) => Ok(remove_column(tbl, name)),
+        Operation::ChangeColumn(tbl, old, new) => change_column(current, tbl, old, Some(new)),
     }
 }
 
-fn create_table(table: &ATable) -> Result<String> {
+fn create_table(table: &ATable, allow_exists: bool) -> Result<String> {
     let coldefs = table
         .columns
         .iter()
         .map(define_column)
         .collect::<Result<Vec<String>>>()?
         .join(",\n");
-    Ok(format!("CREATE TABLE {} (\n{}\n);", table.name, coldefs))
+    let modifier = if allow_exists { "IF NOT EXISTS " } else { "" };
+    Ok(format!(
+        "CREATE TABLE {}{} (\n{}\n);",
+        modifier, table.name, coldefs
+    ))
 }
 
 fn define_column(col: &AColumn) -> Result<String> {
@@ -631,8 +641,8 @@ fn change_column(
         None => new_table.remove_column(old.name()),
     }
     let stmts: [&str; 4] = [
-        &create_table(&new_table)?,
-        &copy_table(&old_table, &new_table),
+        &create_table(&new_table, false)?,
+        &copy_table(old_table, &new_table),
         &drop_table(&old_table.name),
         &format!("ALTER TABLE {} RENAME TO {};", &new_table.name, tbl_name),
     ];
