@@ -110,6 +110,7 @@ impl ConnectionMethods for rusqlite::Connection {
         columns: &'b [Column],
         expr: Option<BoolExpr>,
         limit: Option<i32>,
+        offset: Option<i32>,
         order: Option<&[Order]>,
     ) -> Result<RawQueryResult<'a>> {
         let mut sqlquery = String::new();
@@ -133,6 +134,16 @@ impl ConnectionMethods for rusqlite::Connection {
             helper::sql_limit(limit, &mut sqlquery)
         }
 
+        if let Some(offset) = offset {
+            if limit == None {
+                // Sqlite only supports offset in conjunction with
+                // limit, so add a max limit if we don't have one
+                // already.
+                helper::sql_limit(i32::MAX, &mut sqlquery)
+            }
+            helper::sql_offset(offset, &mut sqlquery)
+        }
+
         debug!("query sql {}", sqlquery);
 
         let stmt = self.prepare(&sqlquery)?;
@@ -164,7 +175,7 @@ impl ConnectionMethods for rusqlite::Connection {
                 table
             ),
             [],
-            |row| sql_val_from_rusqlite(row.get_ref_unwrap(0), &pkcol),
+            |row| sql_val_from_rusqlite(row.get_ref_unwrap(0), pkcol),
         )?;
         Ok(pk)
     }
@@ -302,7 +313,7 @@ fn sqlvalref_to_sqlite<'a, 'b>(valref: &'b SqlValRef<'a>) -> rusqlite::types::To
         BigInt(i) => Owned(Value::Integer(*i)),
         Real(r) => Owned(Value::Real(*r)),
         Text(t) => Borrowed(ValueRef::Text(t.as_bytes())),
-        Blob(b) => Borrowed(ValueRef::Blob(&b)),
+        Blob(b) => Borrowed(ValueRef::Blob(b)),
         #[cfg(feature = "datetime")]
         Timestamp(dt) => {
             let f = dt.format(SQLITE_DT_FORMAT);
@@ -426,22 +437,24 @@ fn sql_valref_from_rusqlite<'a>(
 
 fn sql_for_op(current: &mut ADB, op: &Operation) -> Result<String> {
     match op {
-        Operation::AddTable(table) => Ok(create_table(&table)),
-        Operation::RemoveTable(name) => Ok(drop_table(&name)),
-        Operation::AddColumn(tbl, col) => add_column(&tbl, &col),
-        Operation::RemoveColumn(tbl, name) => Ok(remove_column(current, &tbl, &name)),
-        Operation::ChangeColumn(tbl, old, new) => Ok(change_column(current, &tbl, &old, Some(new))),
+        Operation::AddTable(table) => Ok(create_table(table, false)),
+        Operation::AddTableIfNotExists(table) => Ok(create_table(table, true)),
+        Operation::RemoveTable(name) => Ok(drop_table(name)),
+        Operation::AddColumn(tbl, col) => add_column(tbl, col),
+        Operation::RemoveColumn(tbl, name) => Ok(remove_column(current, tbl, name)),
+        Operation::ChangeColumn(tbl, old, new) => Ok(change_column(current, tbl, old, Some(new))),
     }
 }
 
-fn create_table(table: &ATable) -> String {
+fn create_table(table: &ATable, allow_exists: bool) -> String {
     let coldefs = table
         .columns
         .iter()
         .map(define_column)
         .collect::<Vec<String>>()
         .join(",\n");
-    format!("CREATE TABLE {} (\n{}\n);", table.name, coldefs)
+    let modifier = if allow_exists { "IF NOT EXISTS " } else { "" };
+    format!("CREATE TABLE {}{} (\n{}\n);", modifier, table.name, coldefs)
 }
 
 fn define_column(col: &AColumn) -> String {
@@ -564,8 +577,8 @@ fn change_column(
         None => new_table.remove_column(old.name()),
     }
     let stmts: [&str; 4] = [
-        &create_table(&new_table),
-        &copy_table(&old_table, &new_table),
+        &create_table(&new_table, false),
+        &copy_table(old_table, &new_table),
         &drop_table(&old_table.name),
         &format!("ALTER TABLE {} RENAME TO {};", &new_table.name, tbl_name),
     ];
